import re
from itertools import combinations

punctuations = r"[¥•＂＃＄％＆＇（）＊＋，－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､　、〃〈〉《》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏﹑﹔·'℃°•·．﹑︰〈〉─《﹖﹣﹂﹁﹔！？｡。＂＃＄％＆＇（）＊＋，﹐－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､、〃》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏.．!\"#$%&()*+,\-.\:;<=>?@\[\]\\\/^_`{|}~]"
httplink = r"(https|http)[:\/0-9a-zA-Z.?=&;]*"
unused = r":.+:|<br>|\\n|\(\w+\)|\[.+\]"
simplify_zh_list = ['㐷', '㐹', '㐽', '㑇', '㑈', '㑔', '㑩', '㓆', '㓥', '㓰', '㔉', '㖊', '㖞', '㘎', '㚯', '㛀',
                    '㛟', '㛠', '㛣', '㛤', '㛿', '㟆', '㟜', '㟥', '㡎', '㤘', '㤽', '㥪', '㧏', '㧐', '㧑', '㧟',
                    '㧰', '㨫', '㭎', '㭏', '㭣', '㭤', '㭴', '㱩', '㱮', '㲿', '㳔', '㳕', '㳠', '㳡', '㳢', '㳽',
                    '㴋', '㶉', '㶶', '㶽', '㺍', '㻅', '㻏', '㻘', '䀥', '䁖', '䂵', '䃅', '䅉', '䅟', '䅪', '䇲',
                    '䉤', '䌶', '䌷', '䌸', '䌹', '䌺', '䌻', '䌼', '䌽', '䌾', '䌿', '䍀', '䍁', '䍠', '䎬', '䏝',
                    '䑽', '䓓', '䓕', '䓖', '䓨', '䗖', '䘛', '䘞', '䙊', '䙌', '䙓', '䜣', '䜤', '䜥', '䜧', '䜩',
                    '䝙', '䞌', '䞍', '䞎', '䞐', '䟢', '䢀', '䢁', '䢂', '䥺', '䥽', '䥾', '䥿', '䦀', '䦁', '䦂',
                    '䦃', '䦅', '䦆', '䦶', '䦷', '䩄', '䭪', '䯃', '䯄', '䯅', '䲝', '䲞', '䲟', '䲠', '䲡', '䲢',
                    '䲣', '䴓', '䴔', '䴕', '䴖', '䴗', '䴘', '䴙', '䶮', '万', '与', '丑', '专', '业', '丛', '东',
                    '丝', '丢', '两', '严', '丧', '个', '丰', '临', '为', '丽', '举', '么', '义', '乌', '乐', '乔',
                    '习', '乡', '书', '买', '乱', '了', '争', '于', '亏', '云', '亘', '亚', '产', '亩', '亲', '亵',
                    '亸', '亿', '仅', '仆', '仇', '从', '仑', '仓', '仪', '们', '价', '仿', '众', '优', '伙', '会',
                    '伛', '伞', '伟', '传', '伡', '伣', '伤', '伥', '伦', '伧', '伪', '伫', '体', '余', '佛', '佣',
                    '佥', '侠', '侣', '侥', '侦', '侧', '侨', '侩', '侪', '侬', '侭', '俊', '俣', '俦', '俨', '俩',
                    '俪', '俫', '俭', '修', '借', '债', '倾', '偬', '偻', '偾', '偿', '傤', '傥', '傧', '储', '傩',
                    '僵', '儿', '克', '兑', '兖', '党', '兰', '关', '兴', '兹', '养', '兽', '冁', '内', '冈', '册',
                    '写', '军', '农', '冬', '冯', '冲', '决', '况', '冻', '净', '凄', '准', '凉', '凌', '减', '凑',
                    '凛', '几', '凤', '凫', '凭', '凯', '凶', '出', '击', '凿', '刍', '划', '刘', '则', '刚', '创',
                    '删', '别', '刬', '刭', '刮', '制', '刹', '刽', '刾', '刿', '剀', '剂', '剐', '剑', '剥', '剧',
                    '劝', '办', '务', '劢', '动', '励', '劲', '劳', '势', '勋', '勚', '匀', '匦', '匮', '区', '医',
                    '千', '升', '华', '协', '单', '卖', '卜', '占', '卢', '卤', '卧', '卫', '却', '卷', '卺', '厂',
                    '厅', '历', '厉', '压', '厌', '厍', '厐', '厕', '厘', '厢', '厣', '厦', '厨', '厩', '厮', '县',
                    '叁', '参', '叆', '叇', '双', '发', '变', '叙', '叠', '只', '台', '叶', '号', '叹', '叽', '吁',
                    '吃', '合', '吊', '同', '后', '向', '吓', '吕', '吗', '吨', '听', '启', '吴', '呐', '呒', '呓',
                    '呕', '呖', '呗', '员', '呙', '呛', '呜', '周', '咏', '咙', '咛', '咝', '咤', '咨', '咸', '咽',
                    '哄', '响', '哑', '哒', '哓', '哔', '哕', '哗', '哙', '哜', '哝', '哟', '唇', '唛', '唝', '唠',
                    '唡', '唢', '唤', '啧', '啬', '啭', '啮', '啯', '啰', '啴', '啸', '喂', '喷', '喽', '喾', '嗫',
                    '嗳', '嘘', '嘤', '嘱', '噜', '噪', '嚣', '回', '团', '园', '困', '囱', '围', '囵', '国', '图',
                    '圆', '圣', '圹', '场', '坏', '块', '坚', '坛', '坜', '坝', '坞', '坟', '坠', '垄', '垅', '垆',
                    '垒', '垦', '垩', '垫', '垭', '垯', '垱', '垲', '垴', '埘', '埙', '埚', '堑', '堕', '塆', '墙',
                    '壮', '声', '壳', '壶', '壸', '处', '备', '复', '够', '夫', '头', '夸', '夹', '夺', '奁', '奂',
                    '奋', '奖', '奥', '奸', '妆', '妇', '妈', '妩', '妪', '妫', '姗', '姜', '姹', '娄', '娅', '娆',
                    '娇', '娈', '娘', '娱', '娲', '娴', '婳', '婴', '婵', '婶', '媪', '媭', '嫒', '嫔', '嫱', '嬷',
                    '孙', '学', '孪', '宁', '它', '宝', '实', '宠', '审', '宪', '宫', '家', '宽', '宾', '寝', '对',
                    '寻', '导', '寿', '将', '尔', '尘', '尝', '尧', '尴', '尸', '尽', '局', '层', '屃', '屉', '届',
                    '属', '屡', '屦', '屿', '岁', '岂', '岖', '岗', '岘', '岚', '岛', '岩', '岭', '岳', '岽', '岿',
                    '峃', '峄', '峡', '峣', '峤', '峥', '峦', '峰', '崂', '崃', '崄', '崭', '嵘', '嵚', '嵝', '巅',
                    '巨', '巩', '巯', '币', '布', '帅', '师', '帏', '帐', '帘', '帜', '带', '帧', '席', '帮', '帱',
                    '帻', '帼', '幂', '干', '并', '幸', '广', '庄', '庆', '床', '庐', '庑', '库', '应', '庙', '庞',
                    '废', '庵', '庼', '廪', '开', '异', '弃', '弑', '张', '弥', '弦', '弪', '弯', '弹', '强', '归',
                    '当', '录', '彟', '彦', '彨', '彩', '彻', '征', '径', '徕', '御', '忆', '忏', '志', '忧', '念',
                    '忾', '怀', '态', '怂', '怃', '怄', '怅', '怆', '怜', '总', '怼', '怿', '恋', '恒', '恤', '恳',
                    '恶', '恸', '恹', '恺', '恻', '恼', '恽', '悦', '悫', '悬', '悭', '悮', '悯', '惊', '惧', '惨',
                    '惩', '惫', '惬', '惭', '惮', '惯', '愈', '愠', '愤', '愦', '愿', '慑', '慭', '懑', '懒', '懔',
                    '戆', '戋', '戏', '戗', '战', '戚', '戬', '戯', '户', '才', '扎', '扑', '托', '扣', '执', '扩',
                    '扪', '扫', '扬', '扰', '折', '抚', '抛', '抟', '抠', '抡', '抢', '护', '报', '抵', '担', '拐',
                    '拟', '拢', '拣', '拥', '拦', '拧', '拨', '择', '挂', '挚', '挛', '挜', '挝', '挞', '挟', '挠',
                    '挡', '挢', '挣', '挤', '挥', '挦', '挨', '挽', '捝', '捞', '损', '捡', '换', '捣', '据', '掳',
                    '掴', '掷', '掸', '掺', '掼', '揽', '揾', '揿', '搀', '搁', '搂', '搄', '搅', '搜', '携', '摄',
                    '摅', '摆', '摇', '摈', '摊', '撄', '撑', '撵', '撷', '撸', '撺', '擜', '擞', '攒', '敌', '敚',
                    '敛', '敩', '数', '斋', '斓', '斗', '斩', '断', '旋', '无', '旧', '时', '旷', '旸', '昆', '昙',
                    '昵', '昼', '昽', '显', '晋', '晒', '晓', '晔', '晕', '晖', '暂', '暅', '暗', '暧', '曲', '术',
                    '朱', '朴', '机', '杀', '杂', '权', '杆', '杠', '条', '来', '杨', '杩', '杯', '杰', '松', '板',
                    '极', '构', '枞', '枢', '枣', '枥', '枧', '枨', '枪', '枫', '枭', '柜', '柠', '柽', '栀', '栅',
                    '标', '栈', '栉', '栊', '栋', '栌', '栎', '栏', '树', '栖', '栗', '样', '核', '栾', '桠', '桡',
                    '桢', '档', '桤', '桥', '桦', '桧', '桨', '桩', '桪', '梁', '梦', '梼', '梾', '梿', '检', '棁',
                    '棂', '椁', '椝', '椟', '椠', '椢', '椤', '椫', '椭', '椮', '楼', '榄', '榅', '榇', '榈', '榉',
                    '榝', '槚', '槛', '槟', '槠', '横', '樯', '樱', '橥', '橱', '橹', '橼', '檩', '欢', '欤', '欧',
                    '欲', '歼', '殁', '殇', '残', '殒', '殓', '殚', '殡', '殴', '毁', '毂', '毕', '毙', '毡', '毵',
                    '毶', '氇', '气', '氢', '氩', '氲', '汇', '汉', '汤', '汹', '沄', '沈', '沟', '没', '沣', '沤',
                    '沥', '沦', '沧', '沨', '沩', '沪', '沾', '泛', '泞', '注', '泪', '泶', '泷', '泸', '泺', '泻',
                    '泼', '泽', '泾', '洁', '洒', '洼', '浃', '浅', '浆', '浇', '浈', '浉', '浊', '测', '浍', '济',
                    '浏', '浐', '浑', '浒', '浓', '浔', '浕', '涂', '涌', '涚', '涛', '涝', '涞', '涟', '涠', '涡',
                    '涢', '涣', '涤', '润', '涧', '涨', '涩', '淀', '渊', '渌', '渍', '渎', '渐', '渑', '渔', '渖',
                    '渗', '温', '游', '湾', '湿', '溁', '溃', '溅', '溆', '溇', '滗', '滚', '滞', '滟', '滠', '满',
                    '滢', '滤', '滥', '滦', '滨', '滩', '滪', '漓', '潆', '潇', '潋', '潍', '潜', '潴', '澛', '澜',
                    '濑', '濒', '灏', '灭', '灯', '灵', '灶', '灾', '灿', '炀', '炉', '炖', '炜', '炝', '点', '炼',
                    '炽', '烁', '烂', '烃', '烛', '烟', '烦', '烧', '烨', '烩', '烫', '烬', '热', '焕', '焖', '焘',
                    '煴', '熏', '爱', '爷', '牍', '牦', '牵', '牺', '犊', '状', '犷', '犸', '犹', '狈', '狝', '狞',
                    '独', '狭', '狮', '狯', '狰', '狱', '狲', '猃', '猎', '猕', '猡', '猪', '猫', '猬', '献', '獭',
                    '玑', '玙', '玚', '玛', '玩', '玮', '环', '现', '玱', '玺', '珐', '珑', '珰', '珲', '琎', '琏',
                    '琐', '琼', '瑶', '瑷', '瑸', '璇', '璎', '瓒', '瓮', '瓯', '电', '画', '畅', '畴', '疖', '疗',
                    '疟', '疠', '疡', '疬', '疭', '疮', '疯', '疱', '疴', '症', '痈', '痉', '痒', '痖', '痨', '痪',
                    '痫', '痴', '瘅', '瘆', '瘗', '瘘', '瘪', '瘫', '瘾', '瘿', '癞', '癣', '癫', '皂', '皑', '皱',
                    '皲', '盏', '盐', '监', '盖', '盗', '盘', '眍', '眦', '眬', '睁', '睐', '睑', '瞆', '瞒', '瞩',
                    '矩', '矫', '矶', '矾', '矿', '砀', '码', '砖', '砗', '砚', '砜', '砺', '砻', '砾', '础', '硁',
                    '硕', '硖', '硗', '硙', '硚', '确', '硵', '硷', '碍', '碛', '碜', '碱', '礼', '祃', '祎', '祢',
                    '祯', '祷', '祸', '禀', '禄', '禅', '离', '私', '秃', '秆', '秋', '种', '秘', '积', '称', '秽',
                    '秾', '稆', '税', '稣', '稳', '穑', '穞', '穷', '窃', '窍', '窎', '窑', '窜', '窝', '窥', '窦',
                    '窭', '竖', '竞', '笃', '笋', '笔', '笕', '笺', '笼', '笾', '筑', '筚', '筛', '筜', '筝', '筹',
                    '筼', '签', '筿', '简', '箓', '箦', '箧', '箨', '箩', '箪', '箫', '篑', '篓', '篮', '篯', '篱',
                    '簖', '籁', '籴', '类', '籼', '粜', '粝', '粤', '粪', '粮', '粽', '糁', '糇', '糍', '系', '紧',
                    '絷', '緼', '縆', '纟', '纠', '纡', '红', '纣', '纤', '纥', '约', '级', '纨', '纩', '纪', '纫',
                    '纬', '纭', '纮', '纯', '纰', '纱', '纲', '纳', '纴', '纵', '纶', '纷', '纸', '纹', '纺', '纻',
                    '纼', '纽', '纾', '线', '绀', '绁', '绂', '练', '组', '绅', '细', '织', '终', '绉', '绊', '绋',
                    '绌', '绍', '绎', '经', '绐', '绑', '绒', '结', '绔', '绕', '绖', '绗', '绘', '给', '绚', '绛',
                    '络', '绝', '绞', '统', '绠', '绡', '绢', '绣', '绤', '绥', '绦', '继', '绨', '绩', '绪', '绫',
                    '绬', '续', '绮', '绯', '绰', '绱', '绲', '绳', '维', '绵', '绶', '绷', '绸', '绹', '绺', '绻',
                    '综', '绽', '绾', '绿', '缀', '缁', '缂', '缃', '缄', '缅', '缆', '缇', '缈', '缉', '缊', '缋',
                    '缌', '缍', '缎', '缏', '缐', '缑', '缒', '缓', '缔', '缕', '编', '缗', '缘', '缙', '缚', '缛',
                    '缜', '缝', '缞', '缟', '缠', '缡', '缢', '缣', '缤', '缥', '缦', '缧', '缨', '缩', '缪', '缫',
                    '缬', '缭', '缮', '缯', '缰', '缱', '缲', '缳', '缴', '缵', '罂', '网', '罗', '罚', '罢', '罴',
                    '羁', '羟', '羡', '群', '翘', '翙', '翚', '耢', '耧', '耸', '耻', '聂', '聋', '职', '聍', '联',
                    '聩', '聪', '肃', '肠', '肤', '肮', '肴', '肾', '肿', '胀', '胁', '胄', '胆', '背', '胜', '胡',
                    '胧', '胨', '胪', '胫', '胶', '脉', '脍', '脏', '脐', '脑', '脓', '脔', '脚', '脱', '脶', '脸',
                    '腊', '腌', '腘', '腭', '腻', '腼', '腽', '腾', '膑', '膻', '臜', '致', '舆', '舍', '舣', '舰',
                    '舱', '舻', '艰', '艳', '艺', '节', '芈', '芗', '芜', '芦', '芸', '苁', '苇', '苈', '苋', '苌',
                    '苍', '苎', '苏', '苔', '苧', '苹', '范', '茎', '茏', '茑', '茔', '茕', '茧', '荆', '荐', '荙',
                    '荚', '荛', '荜', '荝', '荞', '荟', '荠', '荡', '荣', '荤', '荥', '荦', '荧', '荨', '荩', '荪',
                    '荫', '荬', '荭', '荮', '药', '莅', '莱', '莲', '莳', '莴', '莶', '获', '莸', '莹', '莺', '莼',
                    '萚', '萝', '萤', '营', '萦', '萧', '萨', '葱', '蒀', '蒇', '蒉', '蒋', '蒌', '蒏', '蒙', '蓝',
                    '蓟', '蓠', '蓣', '蓥', '蓦', '蔂', '蔑', '蔷', '蔹', '蔺', '蔼', '蕰', '蕲', '蕴', '薮', '藓',
                    '藴', '蘖', '虏', '虑', '虚', '虫', '虬', '虮', '虱', '虽', '虾', '虿', '蚀', '蚁', '蚂', '蚃',
                    '蚕', '蚝', '蚬', '蛊', '蛎', '蛏', '蛮', '蛰', '蛱', '蛲', '蛳', '蛴', '蜕', '蜗', '蜡', '蝇',
                    '蝈', '蝉', '蝎', '蝼', '蝾', '螀', '螨', '蟏', '衅', '衔', '补', '表', '衬', '衮', '袄', '袅',
                    '袆', '袜', '袭', '袯', '装', '裆', '裈', '裢', '裣', '裤', '裥', '褛', '褴', '襕', '见', '观',
                    '觃', '规', '觅', '视', '觇', '览', '觉', '觊', '觋', '觌', '觍', '觎', '觏', '觐', '觑', '觞',
                    '触', '觯', '訚', '詟', '誉', '誊', '讠', '计', '订', '讣', '认', '讥', '讦', '讧', '讨', '让',
                    '讪', '讫', '讬', '训', '议', '讯', '记', '讱', '讲', '讳', '讴', '讵', '讶', '讷', '许', '讹',
                    '论', '讻', '讼', '讽', '设', '访', '诀', '证', '诂', '诃', '评', '诅', '识', '诇', '诈', '诉',
                    '诊', '诋', '诌', '词', '诎', '诏', '诐', '译', '诒', '诓', '诔', '试', '诖', '诗', '诘', '诙',
                    '诚', '诛', '诜', '话', '诞', '诟', '诠', '诡', '询', '诣', '诤', '该', '详', '诧', '诨', '诩',
                    '诪', '诫', '诬', '语', '诮', '误', '诰', '诱', '诲', '诳', '说', '诵', '诶', '请', '诸', '诹',
                    '诺', '读', '诼', '诽', '课', '诿', '谀', '谁', '谂', '调', '谄', '谅', '谆', '谇', '谈', '谉',
                    '谊', '谋', '谌', '谍', '谎', '谏', '谐', '谑', '谒', '谓', '谔', '谕', '谖', '谗', '谘', '谙',
                    '谚', '谛', '谜', '谝', '谞', '谟', '谠', '谡', '谢', '谣', '谤', '谥', '谦', '谧', '谨', '谩',
                    '谪', '谫', '谬', '谭', '谮', '谯', '谰', '谱', '谲', '谳', '谴', '谵', '谶', '谷', '豮', '贝',
                    '贞', '负', '贠', '贡', '财', '责', '贤', '败', '账', '货', '质', '贩', '贪', '贫', '贬', '购',
                    '贮', '贯', '贰', '贱', '贲', '贳', '贴', '贵', '贶', '贷', '贸', '费', '贺', '贻', '贼', '贽',
                    '贾', '贿', '赀', '赁', '赂', '赃', '资', '赅', '赆', '赇', '赈', '赉', '赊', '赋', '赌', '赍',
                    '赎', '赏', '赐', '赑', '赒', '赓', '赔', '赕', '赖', '赗', '赘', '赙', '赚', '赛', '赜', '赝',
                    '赞', '赟', '赠', '赡', '赢', '赣', '赪', '赵', '赶', '趋', '趱', '趸', '跃', '跄', '跖', '跞',
                    '践', '跶', '跷', '跸', '跹', '跻', '踌', '踪', '踬', '踯', '蹑', '蹒', '蹰', '蹿', '躏', '躜',
                    '躯', '輼', '车', '轧', '轨', '轩', '轪', '轫', '转', '轭', '轮', '软', '轰', '轱', '轲', '轳',
                    '轴', '轵', '轶', '轷', '轸', '轹', '轺', '轻', '轼', '载', '轾', '轿', '辀', '辁', '辂', '较',
                    '辄', '辅', '辆', '辇', '辈', '辉', '辊', '辋', '辌', '辍', '辎', '辏', '辐', '辑', '辒', '输',
                    '辔', '辕', '辖', '辗', '辘', '辙', '辚', '辞', '辟', '辩', '辫', '边', '辽', '达', '迁', '过',
                    '迈', '运', '还', '这', '进', '远', '违', '连', '迟', '迩', '迳', '迹', '适', '选', '逊', '递',
                    '逦', '逻', '遗', '遥', '邓', '邝', '邬', '邮', '邹', '邺', '邻', '郁', '郏', '郐', '郑', '郓',
                    '郦', '郧', '郸', '酂', '酝', '酦', '酱', '酸', '酽', '酾', '酿', '醖', '采', '释', '里', '鉴',
                    '銮', '錾', '钅', '钆', '钇', '针', '钉', '钊', '钋', '钌', '钍', '钎', '钏', '钐', '钑', '钒',
                    '钓', '钔', '钕', '钖', '钗', '钘', '钙', '钚', '钛', '钜', '钝', '钞', '钟', '钠', '钡', '钢',
                    '钣', '钤', '钥', '钦', '钧', '钨', '钩', '钪', '钫', '钬', '钭', '钮', '钯', '钰', '钱', '钲',
                    '钳', '钴', '钵', '钶', '钷', '钸', '钹', '钺', '钻', '钼', '钽', '钾', '钿', '铀', '铁', '铂',
                    '铃', '铄', '铅', '铆', '铇', '铈', '铉', '铊', '铋', '铌', '铍', '铎', '铏', '铐', '铑', '铒',
                    '铓', '铔', '铕', '铖', '铗', '铘', '铙', '铚', '铛', '铜', '铝', '铞', '铟', '铠', '铡', '铢',
                    '铣', '铤', '铥', '铦', '铧', '铨', '铩', '铪', '铫', '铬', '铭', '铮', '铯', '铰', '铱', '铲',
                    '铳', '铴', '铵', '银', '铷', '铸', '铹', '铺', '铻', '铼', '铽', '链', '铿', '销', '锁', '锂',
                    '锃', '锄', '锅', '锆', '锇', '锈', '锉', '锊', '锋', '锌', '锍', '锎', '锏', '锐', '锑', '锒',
                    '锓', '锔', '锕', '锖', '锗', '锘', '错', '锚', '锛', '锜', '锝', '锞', '锟', '锠', '锡', '锢',
                    '锣', '锤', '锥', '锦', '锧', '锨', '锩', '锪', '锫', '锬', '锭', '键', '锯', '锰', '锱', '锲',
                    '锳', '锴', '锵', '锶', '锷', '锸', '锹', '锺', '锻', '锼', '锽', '锾', '锿', '镀', '镁', '镂',
                    '镃', '镄', '镅', '镆', '镇', '镈', '镉', '镊', '镋', '镌', '镍', '镎', '镏', '镐', '镑', '镒',
                    '镓', '镔', '镕', '镖', '镗', '镘', '镙', '镚', '镛', '镜', '镝', '镞', '镟', '镠', '镡', '镢',
                    '镣', '镤', '镥', '镦', '镧', '镨', '镩', '镪', '镫', '镬', '镭', '镮', '镯', '镰', '镱', '镲',
                    '镳', '镴', '镵', '镶', '长', '门', '闩', '闪', '闫', '闬', '闭', '问', '闯', '闰', '闱', '闲',
                    '闳', '间', '闵', '闶', '闷', '闸', '闹', '闺', '闻', '闼', '闽', '闾', '闿', '阀', '阁', '阂',
                    '阃', '阄', '阅', '阆', '阇', '阈', '阉', '阊', '阋', '阌', '阍', '阎', '阏', '阐', '阑', '阒',
                    '阓', '阔', '阕', '阖', '阗', '阘', '阙', '阚', '阛', '队', '阳', '阴', '阵', '阶', '际', '陆',
                    '陇', '陈', '陉', '陕', '陦', '陧', '陨', '险', '随', '隐', '隶', '隽', '难', '雇', '雏', '雕',
                    '雠', '雳', '雾', '霁', '霉', '霡', '霭', '靓', '靔', '静', '面', '靥', '鞑', '鞒', '鞯', '鞲',
                    '韦', '韧', '韨', '韩', '韪', '韫', '韬', '韵', '页', '顶', '顷', '顸', '项', '顺', '须', '顼',
                    '顽', '顾', '顿', '颀', '颁', '颂', '颃', '预', '颅', '领', '颇', '颈', '颉', '颊', '颋', '颌',
                    '颍', '颎', '颏', '颐', '频', '颒', '颓', '颔', '颕', '颖', '颗', '题', '颙', '颚', '颛', '颜',
                    '额', '颞', '颟', '颠', '颡', '颢', '颣', '颤', '颥', '颦', '颧', '风', '飏', '飐', '飑', '飒',
                    '飓', '飔', '飕', '飖', '飗', '飘', '飙', '飚', '飞', '飨', '餍', '饣', '饤', '饥', '饦', '饧',
                    '饨', '饩', '饪', '饫', '饬', '饭', '饮', '饯', '饰', '饱', '饲', '饳', '饴', '饵', '饶', '饷',
                    '饸', '饹', '饺', '饻', '饼', '饽', '饾', '饿', '馀', '馁', '馂', '馃', '馄', '馅', '馆', '馇',
                    '馈', '馉', '馊', '馋', '馌', '馍', '馎', '馏', '馐', '馑', '馒', '馓', '馔', '馕', '马', '驭',
                    '驮', '驯', '驰', '驱', '驲', '驳', '驴', '驵', '驶', '驷', '驸', '驹', '驺', '驻', '驼', '驽',
                    '驾', '驿', '骀', '骁', '骂', '骃', '骄', '骅', '骆', '骇', '骈', '骉', '骊', '骋', '验', '骍',
                    '骎', '骏', '骐', '骑', '骒', '骓', '骔', '骕', '骖', '骗', '骘', '骙', '骚', '骛', '骜', '骝',
                    '骞', '骟', '骠', '骡', '骢', '骣', '骤', '骥', '骦', '骧', '髅', '髋', '髌', '鬓', '鬶', '魇',
                    '魉', '鱼', '鱽', '鱾', '鱿', '鲀', '鲁', '鲂', '鲃', '鲄', '鲅', '鲆', '鲇', '鲈', '鲉', '鲊',
                    '鲋', '鲌', '鲍', '鲎', '鲏', '鲐', '鲑', '鲒', '鲓', '鲔', '鲕', '鲖', '鲗', '鲘', '鲙', '鲚',
                    '鲛', '鲜', '鲝', '鲞', '鲟', '鲠', '鲡', '鲢', '鲣', '鲤', '鲥', '鲦', '鲧', '鲨', '鲩', '鲪',
                    '鲫', '鲬', '鲭', '鲮', '鲯', '鲰', '鲱', '鲲', '鲳', '鲴', '鲵', '鲶', '鲷', '鲸', '鲹', '鲺',
                    '鲻', '鲼', '鲽', '鲾', '鲿', '鳀', '鳁', '鳂', '鳃', '鳄', '鳅', '鳆', '鳇', '鳈', '鳉', '鳊',
                    '鳋', '鳌', '鳍', '鳎', '鳏', '鳐', '鳑', '鳒', '鳓', '鳔', '鳕', '鳖', '鳗', '鳘', '鳙', '鳚',
                    '鳛', '鳜', '鳝', '鳞', '鳟', '鳠', '鳡', '鳢', '鳣', '鳤', '鸟', '鸠', '鸡', '鸢', '鸣', '鸤',
                    '鸥', '鸦', '鸧', '鸨', '鸩', '鸪', '鸫', '鸬', '鸭', '鸮', '鸯', '鸰', '鸱', '鸲', '鸳', '鸴',
                    '鸵', '鸶', '鸷', '鸸', '鸹', '鸺', '鸻', '鸼', '鸽', '鸾', '鸿', '鹀', '鹁', '鹂', '鹃', '鹄',
                    '鹅', '鹆', '鹇', '鹈', '鹉', '鹊', '鹋', '鹌', '鹍', '鹎', '鹏', '鹐', '鹑', '鹒', '鹓', '鹔',
                    '鹕', '鹖', '鹗', '鹘', '鹙', '鹚', '鹛', '鹜', '鹝', '鹞', '鹟', '鹠', '鹡', '鹢', '鹣', '鹤',
                    '鹥', '鹦', '鹧', '鹨', '鹩', '鹪', '鹫', '鹬', '鹭', '鹮', '鹯', '鹰', '鹱', '鹲', '鹳', '鹴',
                    '鹾', '麦', '麸', '麹', '麺', '麽', '黄', '黉', '黡', '黩', '黪', '黾', '鼋', '鼌', '鼍', '鼹',
                    '齐', '齑', '齿', '龀', '龁', '龂', '龃', '龄', '龅', '龆', '龇', '龈', '龉', '龊', '龋', '龌',
                    '龙', '龚', '龛', '龟', '鿎', '鿏', '鿒', '鿔', '𠀾', '𠆲', '𠆿', '𠇹', '𠉂', '𠉗', '𠋆', '𠚳', '𠛅', '𠛆',
                    '𠛾', '𠡠', '𠮶', '𠯟', '𠯠', '𠰱', '𠰷', '𠱞', '𠲥', '𠴛', '𠴢', '𠵸', '𠵾', '𡋀', '𡋗', '𡋤', '𡍣', '𡒄', '𡝠', '𡞋',
                    '𡞱', '𡠟', '𡥧', '𡭜', '𡭬', '𡳃', '𡳒', '𡶴', '𡸃', '𡺃', '𡺄', '𢋈', '𢗓', '𢘙', '𢘝', '𢘞', '𢙏', '𢙐', '𢙑', '𢙒',
                    '𢙓', '𢛯', '𢠁', '𢢐', '𢧐', '𢫊', '𢫞', '𢫬', '𢬍', '𢬦', '𢭏', '𢽾', '𣃁', '𣆐', '𣈣', '𣍨', '𣍯', '𣍰', '𣎑', '𣏢',
                    '𣐕', '𣐤', '𣑶', '𣒌', '𣓿', '𣔌', '𣗊', '𣗋', '𣗙', '𣘐', '𣘓', '𣘴', '𣘷', '𣚚', '𣞎', '𣨼', '𣭤', '𣯣', '𣱝', '𣲗',
                    '𣲘', '𣳆', '𣶩', '𣶫', '𣶭', '𣷷', '𣸣', '𣺼', '𣺽', '𣽷', '𤆡', '𤆢', '𤇃', '𤇄', '𤇭', '𤇹', '𤈶', '𤈷', '𤊀', '𤊰',
                    '𤋏', '𤎺', '𤎻', '𤙯', '𤝢', '𤞃', '𤞤', '𤠋', '𤦀', '𤩽', '𤳄', '𤶊', '𤶧', '𤻊', '𤽯', '𤾀', '𤿲', '𥁢', '𥅘', '𥅴',
                    '𥅿', '𥆧', '𥇢', '𥎝', '𥐟', '𥐯', '𥐰', '𥐻', '𥞦', '𥧂', '𥩟', '𥩺', '𥫣', '𥬀', '𥬞', '𥬠', '𥭉', '𥮋', '𥮜', '𥮾',
                    '𥱔', '𥹥', '𥺅', '𥺇', '𦈈', '𦈉', '𦈋', '𦈌', '𦈎', '𦈏', '𦈐', '𦈑', '𦈒', '𦈓', '𦈔', '𦈕', '𦈖', '𦈗', '𦈘', '𦈙',
                    '𦈚', '𦈛', '𦈜', '𦈝', '𦈞', '𦈟', '𦈠', '𦈡', '𦍠', '𦛨', '𦝼', '𦟗', '𦨩', '𦰏', '𦰴', '𦶟', '𦶻', '𦻕', '𧉐', '𧉞',
                    '𧌥', '𧏖', '𧏗', '𧑏', '𧒭', '𧜭', '𧝝', '𧝧', '𧮪', '𧳕', '𧹑', '𧹒', '𧹓', '𧹔', '𧹕', '𧹖', '𧹗', '𧿈', '𨀁', '𨀱',
                    '𨁴', '𨂺', '𨄄', '𨅛', '𨅫', '𨅬', '𨉗', '𨐅', '𨐆', '𨐇', '𨐈', '𨐉', '𨐊', '𨑹', '𨟳', '𨠨', '𨡙', '𨡺', '𨤰', '𨰾',
                    '𨰿', '𨱀', '𨱁', '𨱂', '𨱃', '𨱄', '𨱅', '𨱆', '𨱇', '𨱈', '𨱉', '𨱊', '𨱋', '𨱌', '𨱍', '𨱎', '𨱏', '𨱐', '𨱑', '𨱒',
                    '𨱓', '𨱔', '𨱕', '𨱖', '𨷿', '𨸀', '𨸁', '𨸂', '𨸃', '𨸄', '𨸅', '𨸆', '𨸇', '𨸉', '𨸊', '𨸋', '𨸌', '𨸎', '𨸘', '𨸟',
                    '𩏼', '𩏽', '𩏾', '𩏿', '𩐀', '𩓋', '𩖕', '𩖖', '𩖗', '𩙥', '𩙦', '𩙧', '𩙨', '𩙩', '𩙪', '𩙫', '𩙬', '𩙭', '𩙮', '𩙯',
                    '𩙰', '𩟿', '𩠀', '𩠁', '𩠂', '𩠃', '𩠅', '𩠆', '𩠇', '𩠈', '𩠉', '𩠊', '𩠋', '𩠌', '𩠎', '𩠏', '𩠠', '𩡖', '𩧦', '𩧨',
                    '𩧩', '𩧪', '𩧫', '𩧬', '𩧭', '𩧮', '𩧯', '𩧰', '𩧱', '𩧲', '𩧳', '𩧴', '𩧵', '𩧶', '𩧸', '𩧺', '𩧻', '𩧼', '𩧿', '𩨀',
                    '𩨁', '𩨂', '𩨃', '𩨄', '𩨅', '𩨆', '𩨇', '𩨈', '𩨉', '𩨊', '𩨋', '𩨌', '𩨍', '𩨎', '𩨏', '𩨐', '𩩈', '𩬣', '𩬤', '𩭹',
                    '𩯒', '𩰰', '𩲒', '𩴌', '𩽹', '𩽺', '𩽻', '𩽼', '𩽽', '𩽾', '𩽿', '𩾁', '𩾂', '𩾃', '𩾄', '𩾅', '𩾆', '𩾇', '𩾈', '𩾊',
                    '𩾋', '𩾌', '𩾎', '𪉂', '𪉃', '𪉄', '𪉅', '𪉆', '𪉈', '𪉉', '𪉊', '𪉋', '𪉌', '𪉍', '𪉎', '𪉏', '𪉐', '𪉑', '𪉒', '𪉔',
                    '𪉕', '𪎈', '𪎉', '𪎊', '𪎋', '𪎌', '𪑅', '𪔭', '𪚏', '𪚐', '𪜎', '𪞝', '𪟎', '𪟝', '𪠀', '𪠟', '𪠡', '𪠳', '𪠵', '𪠸',
                    '𪠺', '𪠽', '𪡀', '𪡃', '𪡋', '𪡏', '𪡛', '𪡞', '𪡺', '𪢌', '𪢐', '𪢒', '𪢕', '𪢖', '𪢠', '𪢮', '𪢸', '𪣆', '𪣒', '𪣻',
                    '𪤄', '𪤚', '𪥠', '𪥫', '𪥰', '𪥿', '𪧀', '𪧘', '𪨊', '𪨗', '𪨧', '𪨩', '𪨶', '𪨷', '𪨹', '𪩇', '𪩎', '𪩘', '𪩛', '𪩷',
                    '𪩸', '𪪏', '𪪑', '𪪞', '𪪴', '𪪼', '𪫌', '𪫡', '𪫷', '𪫺', '𪬚', '𪬯', '𪭝', '𪭢', '𪭧', '𪭯', '𪭵', '𪭾', '𪮃', '𪮋',
                    '𪮖', '𪮳', '𪮶', '𪯋', '𪰶', '𪱥', '𪱷', '𪲎', '𪲔', '𪲛', '𪲮', '𪳍', '𪳗', '𪴙', '𪵑', '𪵣', '𪵱', '𪶄', '𪶒', '𪶮',
                    '𪷍', '𪷽', '𪸕', '𪸩', '𪹀', '𪹠', '𪹳', '𪹹', '𪺣', '𪺪', '𪺭', '𪺷', '𪺸', '𪺻', '𪺽', '𪻐', '𪻨', '𪻲', '𪻺', '𪼋',
                    '𪼴', '𪽈', '𪽝', '𪽪', '𪽭', '𪽮', '𪽴', '𪽷', '𪾔', '𪾢', '𪾣', '𪾦', '𪾸', '𪿊', '𪿞', '𪿫', '𪿵', '𫀌', '𫀓', '𫀨',
                    '𫀬', '𫀮', '𫁂', '𫁟', '𫁡', '𫁱', '𫁲', '𫁳', '𫁷', '𫁺', '𫂃', '𫂆', '𫂈', '𫂖', '𫂿', '𫃗', '𫄙', '𫄚', '𫄛', '𫄜',
                    '𫄝', '𫄞', '𫄟', '𫄠', '𫄡', '𫄢', '𫄣', '𫄤', '𫄥', '𫄦', '𫄧', '𫄨', '𫄩', '𫄪', '𫄫', '𫄬', '𫄭', '𫄮', '𫄯', '𫄰',
                    '𫄱', '𫄲', '𫄳', '𫄴', '𫄵', '𫄶', '𫄷', '𫄸', '𫄹', '𫅅', '𫅗', '𫅥', '𫅭', '𫅼', '𫆏', '𫆝', '𫆫', '𫇘', '𫇛', '𫇪',
                    '𫇭', '𫇴', '𫇽', '𫈉', '𫈎', '𫈟', '𫈵', '𫉁', '𫉄', '𫊪', '𫊮', '𫊸', '𫊹', '𫊻', '𫋇', '𫋌', '𫋲', '𫋷', '𫋹', '𫋻',
                    '𫌀', '𫌇', '𫌋', '𫌨', '𫌪', '𫌫', '𫌬', '𫌭', '𫌯', '𫍐', '𫍙', '𫍚', '𫍛', '𫍜', '𫍝', '𫍞', '𫍟', '𫍠', '𫍡', '𫍢',
                    '𫍣', '𫍤', '𫍥', '𫍦', '𫍧', '𫍨', '𫍩', '𫍪', '𫍫', '𫍬', '𫍭', '𫍮', '𫍯', '𫍰', '𫍱', '𫍲', '𫍳', '𫍴', '𫍵', '𫍶',
                    '𫍷', '𫍸', '𫍹', '𫍺', '𫍻', '𫍼', '𫍽', '𫍾', '𫍿', '𫎆', '𫎌', '𫎦', '𫎧', '𫎨', '𫎩', '𫎪', '𫎫', '𫎬', '𫎭', '𫎱',
                    '𫎳', '𫎸', '𫎺', '𫏃', '𫏆', '𫏋', '𫏌', '𫏐', '𫏑', '𫏕', '𫏞', '𫏨', '𫐄', '𫐅', '𫐆', '𫐇', '𫐈', '𫐉', '𫐊', '𫐋',
                    '𫐌', '𫐍', '𫐎', '𫐏', '𫐐', '𫐑', '𫐒', '𫐓', '𫐔', '𫐕', '𫐖', '𫐗', '𫐘', '𫐙', '𫐷', '𫑘', '𫑡', '𫑷', '𫓥', '𫓦',
                    '𫓧', '𫓨', '𫓩', '𫓪', '𫓫', '𫓬', '𫓭', '𫓮', '𫓯', '𫓰', '𫓱', '𫓲', '𫓳', '𫓴', '𫓵', '𫓶', '𫓷', '𫓸', '𫓹', '𫓺',
                    '𫓻', '𫓼', '𫓽', '𫓾', '𫓿', '𫔀', '𫔁', '𫔂', '𫔃', '𫔄', '𫔅', '𫔆', '𫔇', '𫔈', '𫔉', '𫔊', '𫔋', '𫔌', '𫔍', '𫔎',
                    '𫔏', '𫔐', '𫔑', '𫔒', '𫔓', '𫔔', '𫔕', '𫔖', '𫔭', '𫔮', '𫔯', '𫔰', '𫔲', '𫔴', '𫔵', '𫔶', '𫔽', '𫕚', '𫕥', '𫕨',
                    '𫖃', '𫖅', '𫖇', '𫖑', '𫖒', '𫖓', '𫖔', '𫖕', '𫖖', '𫖪', '𫖫', '𫖬', '𫖭', '𫖮', '𫖯', '𫖰', '𫖱', '𫖲', '𫖳', '𫖴',
                    '𫖵', '𫖶', '𫖷', '𫖸', '𫖹', '𫖺', '𫗇', '𫗈', '𫗉', '𫗊', '𫗋', '𫗚', '𫗞', '𫗟', '𫗠', '𫗡', '𫗢', '𫗣', '𫗤', '𫗥',
                    '𫗦', '𫗧', '𫗨', '𫗩', '𫗪', '𫗫', '𫗬', '𫗭', '𫗮', '𫗯', '𫗰', '𫗱', '𫗳', '𫗴', '𫗵', '𫘛', '𫘜', '𫘝', '𫘞', '𫘟',
                    '𫘠', '𫘡', '𫘣', '𫘤', '𫘥', '𫘦', '𫘧', '𫘨', '𫘩', '𫘪', '𫘫', '𫘬', '𫘭', '𫘮', '𫘯', '𫘰', '𫘱', '𫘽', '𫙂', '𫚈',
                    '𫚉', '𫚊', '𫚋', '𫚌', '𫚍', '𫚎', '𫚏', '𫚐', '𫚑', '𫚒', '𫚓', '𫚔', '𫚕', '𫚖', '𫚗', '𫚘', '𫚙', '𫚚', '𫚛', '𫚜',
                    '𫚝', '𫚞', '𫚟', '𫚠', '𫚡', '𫚢', '𫚣', '𫚤', '𫚥', '𫚦', '𫚧', '𫚨', '𫚩', '𫚪', '𫚫', '𫚬', '𫚭', '𫛚', '𫛛', '𫛜',
                    '𫛝', '𫛞', '𫛟', '𫛠', '𫛡', '𫛢', '𫛣', '𫛤', '𫛥', '𫛦', '𫛧', '𫛨', '𫛩', '𫛪', '𫛫', '𫛬', '𫛭', '𫛮', '𫛯', '𫛰',
                    '𫛱', '𫛲', '𫛳', '𫛴', '𫛵', '𫛶', '𫛷', '𫛸', '𫛹', '𫛺', '𫛻', '𫛼', '𫛽', '𫛾', '𫜀', '𫜁', '𫜂', '𫜃', '𫜄', '𫜅',
                    '𫜊', '𫜑', '𫜒', '𫜓', '𫜔', '𫜕', '𫜙', '𫜟', '𫜨', '𫜩', '𫜪', '𫜫', '𫜬', '𫜭', '𫜮', '𫜯', '𫜰', '𫜲', '𫜳', '𫝈',
                    '𫝋', '𫝦', '𫝧', '𫝨', '𫝩', '𫝪', '𫝫', '𫝬', '𫝭', '𫝮', '𫝵', '𫞅', '𫞗', '𫞚', '𫞛', '𫞝', '𫞠', '𫞡', '𫞢', '𫞣',
                    '𫞥', '𫞦', '𫞧', '𫞨', '𫞩', '𫞷', '𫟃', '𫟄', '𫟅', '𫟆', '𫟇', '𫟑', '𫟕', '𫟞', '𫟟', '𫟠', '𫟡', '𫟢', '𫟤', '𫟥',
                    '𫟦', '𫟫', '𫟬', '𫟲', '𫟳', '𫟴', '𫟵', '𫟶', '𫟷', '𫟸', '𫟹', '𫟺', '𫟻', '𫟼', '𫟽', '𫟾', '𫟿', '𫠀', '𫠁', '𫠂',
                    '𫠅', '𫠆', '𫠇', '𫠈', '𫠊', '𫠋', '𫠌', '𫠏', '𫠐', '𫠑', '𫠒', '𫠖', '𫠜', '𫢸', '𫧃', '𫧮', '𫫇', '𫬐', '𫭟', '𫭢',
                    '𫭼', '𫮃', '𫰛', '𫵷', '𫶇', '𫷷', '𫸩', '𬀩', '𬀪', '𬂩', '𬃊', '𬇕', '𬇙', '𬇹', '𬉼', '𬊈', '𬊤', '𬍛', '𬍡', '𬍤',
                    '𬒈', '𬒗', '𬕂', '𬘓', '𬘘', '𬘡', '𬘩', '𬘫', '𬘬', '𬘭', '𬘯', '𬙂', '𬙊', '𬙋', '𬜬', '𬜯', '𬞟', '𬟁', '𬟽', '𬣙',
                    '𬣞', '𬣡', '𬣳', '𬤇', '𬤊', '𬤝', '𬨂', '𬨎', '𬩽', '𬪩', '𬬩', '𬬭', '𬬮', '𬬱', '𬬸', '𬬹', '𬬻', '𬬿', '𬭁', '𬭊',
                    '𬭎', '𬭚', '𬭛', '𬭤', '𬭩', '𬭬', '𬭭', '𬭯', '𬭳', '𬭶', '𬭸', '𬭼', '𬮱', '𬮿', '𬯀', '𬯎', '𬱖', '𬱟', '𬳵', '𬳶',
                    '𬳽', '𬳿', '𬴂', '𬴃', '𬴊', '𬶋', '𬶍', '𬶏', '𬶐', '𬶟', '𬶠', '𬶨', '𬶭', '𬶮', '𬷕', '𬸘', '𬸚', '𬸣', '𬸦', '𬸪',
                    '𬸯', '𬹼', '𬺈', '𬺓', '𰬸', '𰰨', '𰶎', '𰾄', '𰾭', '𱊜']


def clean_all(string):
    return clean_unused_tag(clean_httplink(clean_htmlelement(string)))


def clean_unused_tag(string):
    p = re.compile(unused)
    return p.sub(' ', string.strip()).strip()


def clean_htmlelement(data):
    p = re.compile(r'<.*?>')
    return p.sub(' ', data.strip()).strip()


def clean_httplink(string):
    p = re.compile(httplink)
    return p.sub(' ', string.strip()).strip()


def split_lines_by_punc(lines, max_len=50):
    sentences = []
    for line in lines:
        line = full2half(line.strip())
        for sentence in filter(None, re.split(punctuations, line)):
            sentence = sentence.strip()
            if len(sentence) > 0 and len(split_sentence_to_array(sentence)) < max_len:
                sentences.append(sentence)
    return sentences


def split_sentence_to_ngram(sentence, max_len=15):
    ngrams = []
    regex = r"[0-9]+|[a-zA-Z]+\'*[a-z]*|[\w]"
    path = re.findall(regex, sentence, re.UNICODE)
    for i in range(len(path)):
        for j in range(1, min(len(path) + 1, max_len)):
            if i + j <= len(path):
                ngrams.append(join_words_to_sentence([subPath for subPath in path[i:i + j]]))
    return ngrams


def split_sentence_to_ngram_in_part(sentence, max_len=15):
    ngrams = []
    regex = r"[0-9]+|[a-zA-Z]+\'*[a-z]*|[\w]"
    path = re.findall(regex, sentence, re.UNICODE)
    for i in range(len(path)):
        part = []
        for j in range(1, min(len(path) + 1, max_len)):
            if i + j <= len(path):
                part.append(join_words_to_sentence([subPath for subPath in path[i:i + j]]))

        ngrams.append(part)
    return ngrams


def split_text_in_all_comb(sentence):
    result = []
    for i in range(len(sentence)):
        ipart = "".join(sentence[:i + 1])
        irest = sentence[i + 1:]
        if len(irest) > len(ipart):
            for j in split_text_in_all_comb(irest):
                result.append((ipart + " " + j).rstrip())
        else:
            result.append((ipart + " " + join_words_to_sentence(irest)).rstrip())
    return result


def split_sentence_to_array(sentence, merge_non_eng=False):
    sentence = "".join(
        (char if char.isalpha() or char == " " else " " + char + " ") for char in sentence)  # separate punctuation
    if merge_non_eng:
        regex = r"[0-9]|[a-zA-Z]+\'*[a-z]*|[\w]+|[^ ]"
    else:
        regex = r"[0-9]|[a-zA-Z]+\'*[a-z]*|[\w]|[^ ]"
    path = re.findall(regex, sentence, re.UNICODE)
    return path


def join_words_to_sentence(array):
    return ''.join(
        [(' ' if i > 0 and is_all_english(str(subPath)) and not is_all_english(str(array[i - 1])) else '') + subPath + (
            ' ' if is_all_english(str(subPath)) else '') for i, subPath in enumerate(array)]).strip()


def passage_into_chunk(text, length):
    lines = (i.strip() for i in text.splitlines())
    result = []
    chunk = ''
    for line in lines:
        chunk += (line + " ")
        if len(chunk) > length:
            result.append(chunk)
            chunk = ''
    if len(chunk) > 0:
        result.append(chunk)
    return result


def is_all_english(s):
    try:
        s.encode(encoding='utf-8').decode('ascii')
    except:
        return False
    else:
        return True


def is_contain_number(s):
    return bool(re.search(r'[0-9]+', s))


def is_contain_english(s):
    return bool(re.search(r'[a-zA-Z]+', s))


def is_list_contain_string(f_str, f_list):
    return any(s in f_str for s in f_list) or any(f_str in s for s in f_list)


def full2half(s):
    n = []
    for char in s:
        num = ord(char)
        if num == 0x3000:
            num = 32
        elif 0xFF01 <= num <= 0xFF5E:
            num -= 0xfee0
        num = chr(num)
        n.append(num)
    return ''.join(n)


def half2full(s):
    n = []
    for char in s:
        num = ord(char)
        if num == 320:
            num = 0x3000
        elif 0x21 <= num <= 0x7E:
            num += 0xfee0
        num = chr(num)
        n.append(num)
    return ''.join(n)


def sliding_windows(seq, slide=128, append_seq=[]):
    windows_list = []
    slide_pos = []
    seq_length = len(seq)
    windows_min_size = min(slide, seq_length)
    for i in range(int(seq_length / slide) + 1):
        for end in [min(i * slide + slide, seq_length),
                    min(i * slide + slide - int(slide / 2), seq_length)]:
            start = max(0, end - slide)
            slide_seq = seq[start:end] + append_seq
            if len(
                    slide_seq) >= windows_min_size and slide_seq not in windows_list:  # avoid adding same list repeatedly
                windows_list.append(slide_seq)
                slide_pos.append([start, end])
    return windows_list, slide_pos


def jaccard_similarity(list1, list2):
    s1 = set(list1)
    s2 = set(list2)
    return len(s1.intersection(s2)) / len(s1.union(s2))


def is_jaccard_similar(s, t, threshold=0.5):
    return jaccard_similarity(s, t) > threshold


def filter_jaccard_similar_text_from_list(lst, max_return, threshold=0.5):
    while True:
        filtered = False
        for s, t in combinations(lst, 2):
            if is_jaccard_similar(s[0], t[0], threshold) and len(lst) - 1 >= max_return:
                lst.remove(t)
                filtered = True
                break
        if not filtered:
            break
    return lst


def simplify_chinese_rate(sent):
    count = 0
    total = 0
    for i in split_sentence_to_array(sent):
        if i in simplify_zh_list:
            count += 1
        total += 1

    return count / total
